package net.esper.eql.join.table;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import net.esper.event.EventType;
import net.esper.event.EventBean;
import net.esper.collection.MultiKeyUntyped;
import net.esper.util.JavaClassHelper;

import java.util.Set;

/**
 * Index that organizes events by the event property values into hash buckets. Based on a HashMap
 * with {@link net.esper.collection.MultiKeyUntyped} keys that store the property values.
 * <p>
 * Performs coercion of the index keys before storing the keys, and coercion of the lookup keys before lookup.
 * <p>
 * Takes a list of property names as parameter. Doesn't care which event type the events have as long as the properties
 * exist. If the same event is added twice, the class throws an exception on add.
 */
public class PropertyIndTableCoerceAll extends PropertyIndTableCoerceAdd
{
    private static Log log = LogFactory.getLog(PropertyIndTableCoerceAll.class);
    private final Class[] coercionTypes;

    /**
     * Ctor.
     * @param streamNum is the stream number of the indexed stream
     * @param eventType is the event type of the indexed stream
     * @param propertyNames are the property names to get property values
     * @param coercionType are the classes to coerce indexed values to
     */
    public PropertyIndTableCoerceAll(int streamNum, EventType eventType, String[] propertyNames, Class[] coercionType)
    {
        super(streamNum, eventType, propertyNames, coercionType);
        this.coercionTypes = coercionType;
    }

    /**
     * Returns the set of events that have the same property value as the given event.
     * @param keys to compare against
     * @return set of events with property value, or null if none found (never returns zero-sized set)
     */
    public Set<EventBean> lookup(Object[] keys)
    {
        for (int i = 0; i < keys.length; i++)
        {
            Class coercionType = coercionTypes[i];
            Object key = keys[i];
            if ((key != null) && (!key.getClass().equals(coercionType)))
            {
                if (key instanceof Number)
                {
                    key = JavaClassHelper.coerceBoxed((Number) key, coercionTypes[i]);
                    keys[i] = key;
                }
            }
        }
        MultiKeyUntyped key = new MultiKeyUntyped(keys);
        Set<EventBean> events = propertyIndex.get(key);
        return events;
    }

}
