/**************************************************************************************
 * Copyright (C) 2006 Esper Team. All rights reserved.                                *
 * http://esper.codehaus.org                                                          *
 * ---------------------------------------------------------------------------------- *
 * The software in this package is published under the terms of the GPL license       *
 * a copy of which has been included with this distribution in the license.txt file.  *
 **************************************************************************************/
package net.esper.filter;

import net.esper.eql.expression.ExprNode;
import net.esper.event.EventBean;

/**
 * Adapter for use by {@link FilterParamIndexBooleanExpr} to evaluate boolean expressions, providing
 * events per stream to expression nodes. Generated by @{link FilterSpecParamExprNode} for
 * boolean expression filter parameters.
 */
public class ExprNodeAdapter
{
    private final ExprNode exprNode;
    private final EventBean[] prototype;

    private ThreadLocal<EventBean[]> arrayPerThread = new ThreadLocal<EventBean[]>()
    {
        protected synchronized EventBean[] initialValue()
        {
            EventBean[] eventsPerStream = new EventBean[prototype.length];
            System.arraycopy(prototype, 0, eventsPerStream, 0, prototype.length);
            return eventsPerStream;
        }
    };

    /**
     * Ctor.
     * @param exprNode is the boolean expression
     * @param prototype is the row of events the we are matching on
     */
    public ExprNodeAdapter(ExprNode exprNode, EventBean[] prototype)
    {
        this.exprNode = exprNode;
        if (prototype == null)
        {
            this.prototype = new EventBean[1];
        }
        else
        {
            this.prototype = prototype;
        }
    }

    /**
     * Evaluate the boolean expression given the event as a stream zero event.
     * @param event is the stream zero event (current event)
     * @return boolean result of the expression
     */
    public boolean evaluate(EventBean event)
    {
        EventBean[] eventsPerStream = arrayPerThread.get();
        eventsPerStream[0] = event;
        return (Boolean) exprNode.evaluate(eventsPerStream, true);
    }
}
