package net.esper.util;

import java.io.InputStream;

/**
 * Input stream that relies on a simple byte array, unchecked.
 */
public class SimpleByteArrayInputStream extends InputStream
{
    private byte[] buf = null;
    private int count = 0;
    private int pos = 0;

    /**
     * Ctor.
     * @param buf is the byte buffer
     * @param count is the size of the buffer
     */
    public SimpleByteArrayInputStream(byte[] buf, int count)
    {
        this.buf = buf;
        this.count = count;
    }

    public final int available()
    {
        return count - pos;
    }

    public final int read()
    {
        return (pos < count) ? (buf[pos++] & 0xff) : -1;
    }

    public final int read(byte[] b, int off, int len)
    {
        if (pos >= count)
        {
            return -1;
        }

        if ((pos + len) > count)
        {
            len = (count - pos);
        }

        System.arraycopy(buf, pos, b, off, len);
        pos += len;
        return len;
    }

    public final long skip(long n)
    {
        if ((pos + n) > count)
        {
            n = count - pos;
        }
        if (n < 0)
        {
            return 0;
        }
        pos += n;
        return n;
    }
}
