package net.esper.multithread;

import junit.framework.TestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import net.esper.client.*;
import net.esper.support.bean.SupportBean;
import net.esper.support.util.SupportMTUpdateListener;
import net.esper.event.EventBean;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.Arrays;

/**
 * Test for multithread-safety and deterministic behavior when using insert-into.
 */
public class TestMTDeterminismListener extends TestCase
{
    private static final Log log = LogFactory.getLog(TestMTDeterminismListener.class);
    private EPServiceProvider engine;

    public void tearDown()
    {
        engine.initialize();
    }

    public void testOrderedDelivery() throws Exception
    {
        engine = EPServiceProviderManager.getDefaultProvider();
        engine.initialize();
        trySend(3, 1000);
    }

    public void manualTestOrderedDeliveryFail() throws Exception
    {
        /**
         * Commented out as this is a manual test
         */
        Configuration config = new Configuration();
        config.getEngineDefaults().getThreading().setListenerDispatchPreserveOrder(false);
        engine = EPServiceProviderManager.getDefaultProvider(config);
        engine.initialize();
        trySend(3, 1000);
    }

    private void trySend(int numThreads, int numEvents) throws Exception
    {
        // setup statements
        EPStatement stmtInsert = engine.getEPAdministrator().createEQL("select count(*) as cnt from " + SupportBean.class.getName());
        SupportMTUpdateListener listener = new SupportMTUpdateListener();       
        stmtInsert.addListener(listener);

        // execute
        ExecutorService threadPool = Executors.newFixedThreadPool(numThreads);
        Future future[] = new Future[numThreads];
        for (int i = 0; i < numThreads; i++)
        {
            future[i] = threadPool.submit(new SendEventCallable(i, engine, new GeneratorIterator(numEvents)));
        }

        threadPool.shutdown();
        threadPool.awaitTermination(10, TimeUnit.SECONDS);

        for (int i = 0; i < numThreads; i++)
        {
            assertTrue((Boolean) future[i].get());
        }

        EventBean events[] = listener.getNewDataListFlattened();
        long[] result = new long[events.length];
        for (int i = 0; i < events.length; i++)
        {
            result[i] = (Long) events[i].get("cnt");
        }
        //log.info(".trySend result=" + Arrays.toString(result));

        // assert result
        assertEquals(numEvents * numThreads, events.length);
        for (int i = 0; i < numEvents * numThreads; i++)
        {
            assertEquals(result[i], (long) i + 1);
        }
    }
}
